/* 
 * ==========================================================
 * Artekit demo sketch for the AK-MMA8452 accelerometer
 * breakout board.
 *
 * Copyright (c) 2013 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-MMA8452
 * ==================================
 *   AN5                   SCL
 *   AN4                   SDA
 *   3.3V                  VCC
 *   GND                   GND
 *
 * This example works with the SA0 pin set to GND. If you have
 * this pin to VCC, change the SENSOR_ADDR value to 0x1D.
 * You may use this demo with a logic level converter, because of
 * the 5V on the Arduino I2C lines (this board is not 5V tolerant).
 * There are other ways, like using a non-standard Wire library to
 * remove the pull-up resistors on the I2C lines.
 *
 * ==========================================================
 */
 
#include <Wire.h>

#define SENSOR_ADDR 0x1C

float IntialGs[3];
bool InitialRead = false;

/* This function reads a byte for the given register */
static byte i2cRead(byte reg)
{
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(SENSOR_ADDR, 1);
  return Wire.read();
}

/* This function will read a sequence of registers starting by the 'reg'
 * register, for a quantity of 'qty' registers. On success the 'out' array
 * will contain the read data. This function is used to read the 6
 * acceleration values in one I2C transanction.
 */
static void i2cReadAccelValues(byte reg, byte* out, int qty)
{
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(SENSOR_ADDR, qty);
  
  while(Wire.available() < qty);
  
  for (int i = 0; i < qty; i++)
  {
    out[i] = Wire.read(); 
  }
}
 
/* This function writes a byte to the given register */
static void i2cWrite(byte reg, byte value)
{
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

/* Enters or leaves the stand-by mode. If the 'standby' parameter is
 * true, it will enter stand-by mode. If it is false it will leave
 * stand-by mode.
 */
static void sensorStandBy(bool standby)
{
  byte reg = i2cRead(0x2A);
  i2cWrite(0x2A, standby ? reg & ~1 : reg | 1);
}

static void sensorConfig()
{
  byte reg;
  
  /* Check the WHO_AM_I register */
  reg = i2cRead(0x0D);
  if (reg != 0x2A)
  {
    Serial.print("Wrong sensor ID: ");
    Serial.println(reg, HEX);
    while (1);  /* Do nothing */  
  }
  
  /* Put the sensor in stand-by mode */
  sensorStandBy(true);
  
  /* 2g scale */
  i2cWrite(0x0E, 0x00);
  
  /* 50Hz data rate */
  reg = i2cRead(0x2A);
  i2cWrite(0x2A, reg & ~0x18);
  
  /* Set up interrupt pin (not really used in this demo,
   * just to see it with the DSO).
   */
   
  /* Active-low polarity, push-pull */
  i2cWrite(0x2C, 0x00);
  
  /* Data ready enabled ... */
  i2cWrite(0x2D, 0x01);
  /* ... on INT1 */
  i2cWrite(0x2E, 0x01);
  
  /* Exit stand-by mode */
  sensorStandBy(false);
}

static void sensorRead()
{
  byte reg_data[6];
  short sensorData[3];
  float Gs[3];
  
  /* Read the acceleration data */
  i2cReadAccelValues(0x01, reg_data, 6);

  for (int i = 0; i < 3; i++)
  {
    /* Get the 12-bit value and put it inside a short */
    sensorData[i] = reg_data[i*2] << 4;
    sensorData[i] |= reg_data[(i*2) + 1] >> 4;

    /* If the MSB value is greater than 0x7F, then it is a
     * signed value.
     */
    if (reg_data[i*2] > 0x7F)
    {
      /* Convert it to a signed value */
      sensorData[i] = (~sensorData[i] & 0xFFF) + 1;
      sensorData[i] *= -1;
    }
   
    /* Conver the axis value to Gs */
    Gs[i] = (float) sensorData[i] / ((1 << 12) / (2 * 2));
  }
  
  /* Print the read data */   
  Serial.print(Gs[0], 4);
  Serial.print("\t");
  Serial.print(Gs[1], 4);
  Serial.print("\t");
  Serial.println(Gs[2], 4);
}

void setup()
{
  Wire.begin();
  Serial.begin(57600);
  sensorConfig();  
}

void loop()
{
  /* Read the accelerometer data each 20 ms */
  sensorRead();
  delay(20);
}

